*************************************************************************
*        Routine to smear the resonance mass according  to Breit-Wigner *
*                  CompHEP 4.*  -  PYTHIA 6.2 interface                 *
*************************************************************************
      SUBROUTINE SMEARMASS
*...PYTHIA common blocks
      INCLUDE 'PYTHIA62.inc'        
*...Les Houches Agreement I common blocks
      INCLUDE 'LHA.inc'        
*...CompHEP-PYTHIA interface
      INCLUDE 'COMPHEPINT.inc'
      SAVE

      INTEGER I, J, Q
      INTEGER KCP
      DOUBLE PRECISION PX1(20,5), PX2(20,5), PX3(20,5)
      DOUBLE PRECISION PX, PXX(20,5)
      DOUBLE PRECISION PLAB, ELAB, SHAT, PM2, GAMMA, PM1
      DOUBLE PRECISION PMAV, WIDTH, WBAND, PMUP, PMDN, ATL, ATU, PMOD
      DOUBLE PRECISION E1N, E2, E2N, PMODN, PJL, PJLN

*...some values in lab frame
      PLAB = PUP(3,1) + PUP(3,2)
      ELAB = PUP(4,1) + PUP(4,2)
      SHAT = (ELAB - PLAB) * (ELAB + PLAB)

*...loop over the particles in final state
      DO I = 3, NUP
*...if unstable particle then smear its mass
        KCP = PYCOMP(IDUP(I))
        IF (PMAS(ABS(KCP),2).GT.0.) THEN
*...transfer out-momenta to REAL*8
          DO J = 1, NUP
            DO Q = 1, 5
              PX1(J, Q) = PUP(Q, J)
            ENDDO
          ENDDO

*...invariant mass of the cluster recoiled the Ith particle
          IF (NUP.EQ.4) THEN              ! 2->2 case
            IF (I.eq.3) PM2 = PX1(4,5)
            IF (I.eq.4) PM2 = PX1(3,5)
          ELSE
            PX  = SQRT(PX1(I,1)**2 + PX1(I,2)**2 + (PLAB-PX1(I,3))**2)
            PM2 = (ELAB-PX1(I,4)-PX)*(ELAB-PX1(I,4)+PX)
            IF (PM2.lt.0.) PM2 = abs(PM2)
            PM2 = SQRT(PM2)
          ENDIF
 
*...boost of all momenta to partonic CMS
          GAMMA = 1.d0/(SQRT(ELAB-PLAB)*SQRT(ELAB+PLAB))
          DO J = 1, NUP
            PX2(j,1) = PX1(j,1)
            PX2(j,2) = PX1(j,2)
            PX2(j,3) = GAMMA*(-PLAB*PX1(j,4)+ELAB*PX1(j,3))
            PX2(j,4) = GAMMA*( ELAB*PX1(j,4)-PLAB*PX1(j,3))
            PX2(j,5) = PX1(j,5)
          ENDDO

*...defs for Breit-Wigner measure
          PMAV  = PX2(I,5)
          WIDTH = PMAS(ABS(ABS(KCP)),2)
          WBAND = PMAS(ABS(ABS(KCP)),3)
          PMUP  = WBAND+PMAV
          PMDN  = -WBAND+PMAV
          ATL   = ATAN((PMDN**2-PMAV**2)/(PMAV*WIDTH))
          ATU   = ATAN((PMUP**2-PMAV**2)/(PMAV*WIDTH))

*...BW smearing of the mass of Ith particle
555       PM1 = SQRT(PMAV**2+PMAV*WIDTH*TAN(ATL+PYR(0)*(ATU-ATL)))
          PX3(I,5) = PM1                ! new mass of Ith particle

*...new energy of the Ith particle in partonic CMS
          E1N = (PM1**2-PM2**2 + SHAT)/2.d0/SQRT(SHAT)
          IF (E1N.LE.PM1) GOTO 555   !check masses
          PX3(I,4) = E1N
*...old and new energy of the recoiled cluster in partonic CMS
          E2  = SQRT(SHAT)-PX2(I,4)
          E2N = SQRT(SHAT)-E1N
          IF (E2N.LE.PM2) GOTO 555   !check masses

*...old space momenta of the Ith particle and recoiled cluster in partonic CMS
          PMOD = SQRT(PX2(I,1)**2 + PX2(I,2)**2 + PX2(I,3)**2)	  
*...new space momenta of the Ith particle and recoiled cluster in partonic CMS
          PMODN = SQRT(E1N-PM1)*SQRT(E1N+PM1)
*...new space components of the Ith momentum in partonic CMS via rescaling
          DO Q = 1, 3
            PXX(I, Q) = PX2(I, Q)/PMOD
            PX3(I, Q) = PXX(I, Q)*PMODN
          ENDDO

*...new momentum of the recoiled particle in 2->2 case
          IF (NUP.eq.4) THEN
            IF (I.eq.3) I2 = 4
            IF (I.eq.4) I2 = 3
            PX3(I2,1) = -PX3(I,1)
            PX3(I2,2) = -PX3(I,2)
            PX3(I2,3) = -PX3(I,3)
            PX3(I2,4) = E2N
            PX3(I2,5) = PM2
            GOTO 333
          ENDIF

*...new momenta for other out particles (neq Ith)
          GAMMA = 1.d0/SQRT((E2-PMOD)*(E2N+PMODN))
     &        *1.d0/SQRT((E2+PMOD)*(E2N-PMODN))
          DO J = 3, NUP
            IF (J.NE.I) then
              PJL = -(PX2(j,1)*PXX(I,1)+PX2(j,2)*PXX(I,2)
     &  	+PX2(j,3)*PXX(I,3))
              PJLN = GAMMA*(-(E2N*PMOD-E2*PMODN)*PX2(j,4)
     &  	 +(E2*E2N-PMOD*PMODN)*PJL)
              PX3(j,4) = GAMMA*((E2*E2N-PMOD*PMODN)*PX2(j,4)
     &  	     -(E2N*PMOD-E2*PMODN)*PJL)
              PX3(j,1) = PX2(j,1)-(PJLN-PJL)*PXX(I,1)
              PX3(j,2) = PX2(j,2)-(PJLN-PJL)*PXX(I,2)
              PX3(j,3) = PX2(j,3)-(PJLN-PJL)*PXX(I,3)
              PX3(j,5) = PX2(j,5) 
            ENDIF
          ENDDO

333       CONTINUE
*...boost from partonic CMS to the lab frame
          GAMMA = 1.d0/(SQRT(ELAB-PLAB)*SQRT(ELAB+PLAB))
          DO j=3,NUP
            PUP(1,j) = PX3(j,1)
            PUP(2,j) = PX3(j,2)
            PUP(3,j) = GAMMA*(PLAB*PX3(j,4)+ELAB*PX3(j,3))
            PUP(4,j) = GAMMA*(ELAB*PX3(j,4)+PLAB*PX3(j,3))
            PUP(5,j) = PX3(j,5)
          ENDDO
        ENDIF                     ! IF(PMAS(ABS(KCP),2).gt.0.)
      ENDDO                     ! DO I=3,NUP
      RETURN
      END
