/* 
 * Copyright (C) 2001-2006, CompHEP Collaboration
 *------------------------------------------------------
 * $Id$
 *
 * $Log$
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tag_base.h"

/**************************************************************************************************/
int baselen (tags * b) {
  int i;
  int res = 0;
  int len = 0;
  if (NULL != b) {
    if (NULL != b->basename) len = (strlen(b->basename) + 1) * sizeof(char);
    for (i = 0; i < b->number_of_tags; ++i)
      len += taglen(b->tag[i]);
    res = sizeof(int) + len;
  }
  return res;
}

/**************************************************************************************************/
int taglen (elementary_tag * t) {
  int i;
  int res = 0;
  int len = 0;
  if (NULL != t) {
    if (NULL != t->tagname) len = strlen(t->tagname) + 1;
    for (i = 0; i < t->tagsize; ++i)
      if (NULL != t->commands[i]) len += strlen(t->commands[i]) + 1;
    res = 2 * sizeof(int) + len * sizeof(char);
  }

  return res;
}

/**************************************************************************************************/
int freetag (elementary_tag * t) {
  int i;
  if (NULL != t) {
    if (NULL != t->tagname) free(t->tagname);
    for (i = 0; i < t->tagsize; ++i)
      if (NULL != t->commands[i]) free(t->commands[i]);
    free(t);
  }

  return 1;
}

/**************************************************************************************************/
int comlen (string_comnd * c) {
  int clen = 0, vlen = 0;
  if (NULL != c->name)  clen = strlen(c->name) + 1;
  if (NULL != c->value) vlen = strlen(c->value) + 1;
  return (sizeof(int) + sizeof(char) * (clen + vlen));
}

/**************************************************************************************************/
string_comnd * initcom (string_comnd * c, char * name, char * val, int type) {
  int nlen, vlen;

  if (NULL == name) name = "";
  nlen = strlen (name) + 1;
  if (NULL == val) val = "";
  vlen = strlen (val) + 1;
  if (NULL != c) free(c);
//  c = malloc (sizeof(int) + sizeof(char) * (nlen + vlen));
  c = malloc (sizeof(string_comnd));

  if (NULL != c) {
    c->type = type;
    c->name = malloc (nlen * sizeof(char));
    strcpy(c->name, name);
    c->value = malloc (vlen * sizeof(char));
    strcpy(c->value, val);
  } else {
    fprintf(stderr, "Error! can not assign memory for a command! Stop...");
  }

  return c;
}

/**************************************************************************************************/
elementary_tag * inittag (elementary_tag * t, char * name) {
  int len;
  if (NULL == name) name = "";
  len = strlen(name) + 1;
  if (NULL != t) free(t);
//  t = malloc (2 * sizeof(int) + sizeof(char*) + len * sizeof(char));
  t = malloc (sizeof(elementary_tag));

  if (NULL != t) {
    t->used = 0;
    t->tagsize = 0;
    t->commands = NULL;
    t->tagname = malloc(len * sizeof(char));
    strcpy(t->tagname, name);
  } else {
    fprintf(stderr, "Error! can not assign memory for a tag! Stop...");
  }

  return t;
}

/**************************************************************************************************/
tags * initcap (tags * b, char * name) {
  int len;
  if (NULL == name) name = "";
  len = strlen(name) + 1;
  if (NULL != b) free(b);
//  b = malloc (sizeof(int)  + sizeof(elementary_tag*)+ len * sizeof(char));
  b = malloc (sizeof(tags));

  if (NULL != b) {
    b->number_of_tags = 0;
    b->basename = malloc(len * sizeof(char));
    strcpy(b->basename, name);
    b->tag = NULL;
  }
  return b;
}

/**************************************************************************************************/
int event_parser (char * in, char ** eventword) {
  int i, n = 0;
  int len, fulllen;
  int last = 0;

  fulllen = strlen (in);
  while (last < fulllen) {
    char *st = strchr (in, ':');
    if (!st) {
      fprintf (stderr, "Error! The event does not contain a colon! \n");
      fprintf (stderr, "       Event string: %s\n",in);
      return 0;
    }
    len = fulllen - strlen (st);
    eventword[n] = malloc ((len - last + 1) * sizeof (char));

    for (i = last; i < len; i++)
      eventword[n][i - last] = in[i];
    eventword[n][len - last + 1] = '\0';

    last = len + 1;
    in[len] = '_';

    n++;
  }

  return 1;
}

/**************************************************************************************************/
elementary_tag * add_com (elementary_tag * t, string_comnd * c) {
  char * line = NULL;

  if (NULL == c || NULL == c->name|| NULL == c->value) return t;
  if (2 == c->type) {
    line = malloc((strlen(c->name) + strlen(c->value) + 4) * sizeof(char));
    sprintf (line, "%s='%s'", c->name, c->value);
  } else {
    line = malloc((strlen(c->name) + strlen(c->value) + 2) * sizeof(char));
    sprintf (line, "%s=%s", c->name, c->value);
  }
  if (NULL != line) {
//    t = realloc (t, (strlen(line) + 1) * sizeof(char) + taglen(t));
    t->commands = realloc (t->commands, (t->tagsize + 1) * sizeof (char*));
    t->commands[t->tagsize] = malloc (strlen(line));
    strcpy(t->commands[t->tagsize], line);
    t->tagsize++;
    free(line);
  }
  return t;
}

/**************************************************************************************************/
elementary_tag * remove_com (elementary_tag * t, char * cname) {
/* 
still not implemented
*/
  return t;
}

/**************************************************************************************************/
tags * add_tag (tags * b, elementary_tag * t) {
  int lenbaselen = baselen(b) + taglen(t);
//  b = realloc (b, lenbaselen);
  if (NULL != b) {
    b->tag = realloc ( b->tag, (b->number_of_tags + 1) * sizeof(elementary_tag*));
    b->tag[b->number_of_tags] = tagcpy (b->tag[b->number_of_tags], t);
    b->number_of_tags++;
  } else {
    printf ("beda!!!!! can not add the tag %s to the base %s", t->tagname, b->basename);
  }

  return b;
}

/**************************************************************************************************/
elementary_tag * tagcpy (elementary_tag * src, elementary_tag * dist) {
  int i;

  if (NULL != src) src = NULL;
  src = malloc (taglen(dist));

  src->tagsize = dist->tagsize;
  src->used = dist->used;
  src->tagname = malloc (strlen(dist->tagname) * sizeof(char));
  strcpy (src->tagname, dist->tagname);

  src->commands = malloc (dist->tagsize * sizeof(char*));
  for (i = 0; i < dist->tagsize; ++i) {
    src->commands[i] = calloc (strlen(dist->commands[i]), sizeof(char));
    strcpy (src->commands[i], dist->commands[i]);
  }
  return src;
}

/**************************************************************************************************/
int printtag (FILE * f, elementary_tag * t) {
  int i;

  fprintf (f, "tag name: %s (usage: %i)\n", t->tagname, t->used);
  fprintf (f, "   number of commands: %i\n", t->tagsize);
  for (i = 0; i < t->tagsize; ++i) {
    fprintf (f, "   command %i: %s\n", i+1, t->commands[i]);
  }
  return 1;
}

/**************************************************************************************************/
int printtagbase (FILE * f, tags * b) {
  int i;

  fprintf (f, "****************************************************\n");
  fprintf (f, "tagbase name: %s\n", b->basename);
  fprintf (f, "number of tags: %i\n", b->number_of_tags);
  for (i = 0; i < b->number_of_tags; ++i) {
    printtag (f, b->tag[i]);
  }
  fprintf (f, "****************************************************\n");
  return 1;
}

/**************************************************************************************************/
tags * remove_tag (tags * base, char * tname) {
/* 
still not implemented
*/
  return base;
}
