/* 
   * Copyright (C) 2001-2003, CompHEP Collaboration
   *------------------------------------------------------
   * $Id: tag_reader.c,v 1.8 2003/08/25 09:33:18 sherstnv Exp $
   *
   * $Log: tag_reader.c,v $
   * Revision 1.8  2003/08/25 09:33:18  sherstnv
   * type midstr - middlestr is changed
   *
   * Revision 1.7  2003/06/25 18:46:41  kryukov
   * *** empty log message ***
   *
   * Revision 1.6  2003/06/17 11:15:26  sherstnv
   * An error in memory usage has been fixed.
   *
   * Revision 1.5  2003/04/22 09:01:19  kryukov
   * Apply indent to improve readability of files
   *
   * Revision 1.4  2003/04/20 17:55:02  kryukov
   * Clean up warnings during make
   *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tag_reader.h"
#include "tag_parser.h"

FILE * evfile;

int char_switch = 0;

int format_error;

char new_symbol (FILE * file)
{
  char sym = fgetc (file);

  if (39 == sym) {     /* 39 -> ' */
    char_switch++;
    char_switch = char_switch % 2;
  }

  if (char_switch)
    return sym;
  while (sym == 32 || sym == 10) /*  32 -> ' ' (space), 10 -> line feed  */
      sym = fgetc (file);
  return sym;
}

static string_comnd * command_reader (string_comnd * c)
{
  long i;
  long len;
  char symb;
  char * buffer;

  len = 0;
  symb = new_symbol (evfile);
  while (',' != symb) {
    symb = new_symbol (evfile);
    ++len;
  }
  fseek (evfile, -len, SEEK_CUR);

  buffer = malloc (len * sizeof (char));
  for (i = 0; i < len; i++) {
    buffer[i] = new_symbol (evfile);
  }
  buffer[len] = '\0';

  format_error = 0;
  if (NULL == strrchr(buffer, '=')) format_error = 1;

  return command_parser (buffer, c);
}

static char * tagname_reader (char * tagname)
{
  long i;
  long len;
  char symb;

  len = 0;
  symb = new_symbol (evfile);
  while (':' != symb) {
    symb = new_symbol (evfile);
    ++len;
  }
  fseek (evfile, -len, SEEK_CUR);

  tagname = malloc (len * sizeof (char));
  for (i = 0; i < len; i++) {
    symb = new_symbol (evfile);
    if (symb != ':') {
      tagname[i] = symb;
      tagname[i + 1] = '\0';
    }
  }

  format_error = 0;
  if (0 == strlen(tagname)) format_error = 1;

  return tagname;
}

static elementary_tag * tag_reader (elementary_tag * tag)
{
  int i;
  char symb;
  char * tagname;
  string_comnd * c;

  tagname = tagname_reader (tagname);
  if (1 == format_error) {
    fprintf (stderr, 
    "\n***Error! "
    "function: command_reader: "
    "an error with the tag syntax in a tag %s\n", tagname);
    return tag;
  }
  tag->tagname = realloc (tag->tagname, strlen(tagname) * sizeof (char));
  strcpy (tag->tagname, tagname);

  for (i = 0; i < MAX_COMMAND_NUMBER; i++) {
    symb = new_symbol (evfile);
    if (symb == ';') {
      tag->tagsize = i;
      return tag;
    }
    else
      fseek (evfile, -1, 1);

    c = command_reader (c);
    if (1 == format_error) {
      fprintf (stderr, "\n***Error! "
      "function: command_reader: "
      "an error with the command syntax in a tag %s\n", tag->tagname);
      exit (1);
    }
    tag = add_com (tag, c);
    free (c);
  }

  if (i == MAX_COMMAND_NUMBER)
  fprintf (stderr, "\nWarning: number of commands in the tag %s exceeds limit (%i commands)!\n",
           tag->tagname, MAX_COMMAND_NUMBER);

  symb = new_symbol (evfile);
  while (';' != symb) symb = new_symbol (evfile);
  return tag;
}


extern tags * cup_reader (FILE * inFile, tags * all_tags)
{
  int y;
  int numtag = 0;
  char symbol;

/* check the format:  
   tag -> ##tagname: comname=comval, ..., comname=comval,; 
   Any number of spaces and line-feeds between commands, comnames and comvalues is allowed.
*/
  evfile = inFile;
  for (y = 0;; y++) {
    symbol = new_symbol (evfile);
    if (symbol == '#') {
      symbol = new_symbol (evfile);
      if (symbol != '#') {
        fprintf (stderr, "This file is not in the CompHEP v2 format...");
        return all_tags;
      }
    }
    else {
      fprintf (stderr, "This file is not in the CompHEP v2 format..");
      return all_tags;
    }

/* read tags in the header */
    elementary_tag * tag;
    tag = inittag (tag, "temp");
    tag = tag_reader (tag);
    if (0 == format_error) {
      all_tags->tag[numtag] = tag;
      ++numtag;
      all_tags->tag = realloc (all_tags->tag, (numtag + 1) * sizeof (elementary_tag));
    }
    else
      free (tag);

    symbol = new_symbol (evfile);
    if (symbol == ';' || y > 10000) break;
    else fseek (evfile, -1, SEEK_CUR);
  }
  all_tags->number_of_tags = numtag;
  return all_tags;
}
