#include <ctype.h>
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include"tag_reader.h"
#include"tag_parser.h"
#include"tag_writer.h"
#include"routines.h"
#include"translator.h"

int main(int argc,char ** argv)
{
  int i;
  int nproc;
  int nevents;
  int theformatver;
  double fullcs;
  int * nout;
  char buff[STRSIZE];
  char dump[STRSIZE];
  char genversion[STRSIZE];
  FILE * inFile;
  FILE * outFile;
  string_comnd * c;

  if (strcmp (argv[1], "") == 0 || strcmp (argv[1], "-h") == 0 || argc > 3) {
    ShowHelp (0);
    return 1;
  }
  if ((strcmp (argv[1], "-help") == 0) || (strcmp (argv[1], "--help") == 0)) {
      ShowHelp (1);
      return 1;
  };

  inFile = fopen (argv[1], "r");
  if (!inFile) {
    fprintf (stderr, "***Error! input file not found: %s\n", argv[1]);
    return 2;
  }
  outFile = fopen (argv[2], "w");
  if (!inFile) {
    fprintf (stderr, "***Error! output file can not be opened: %s\n", argv[1]);
    return 2;
  }

/****************************************************************/
/*     initialize global variables                              */

   theformatver = check_format (inFile);
   switch (theformatver) {
     case 1: fprintf (stdout, "The file is in the 1st format. I'll translate it to the 2nd format!\n");
       break;
     case 2: fprintf (stdout, "The file is in the 2st format. Nothing to be done!\n");
       return 0;
       break;
     default: fprintf (stdout, "Unknown format... Stop\n");
       return -1;
   }

  while(NULL != fgets(buff,1000,inFile)) {
    if (NULL != strstr(buff, "#CompHEP") ) {
      sscanf(buff,"#CompHEP version %s%[^a]", genversion, dump);
    }
    if(NULL != strstr(buff, "#Total_cross_section_(pb)")) {
      sscanf(buff,"#Total_cross_section_(pb) = %lf", &fullcs);
      nout = malloc (nproc * sizeof (int));
    }
    if(NULL != strstr(buff, "#Number_of_subprocesses")) {
      sscanf(buff,"#Number_of_subprocesses = %i", &nproc);
      nout = malloc (nproc * sizeof (int));
    }
    if(NULL != strstr(buff, "#Events_mixed_and_randomized")) {
       sscanf(buff,"#Events_mixed_and_randomized = %i", &nevents);
    }
    if(NULL != strstr(buff,"#Nproc")) {
       break;
    }
  }
  rewind (inFile);

  process * proc1;
  proc1 = read_subproc_info (proc1, inFile);
  rewind (inFile);

  beams * thebeams;
  thebeams = fillbeams (thebeams, proc1);
/****************************************************************/
/*         rewriting of header                                  */
  tags * header;
  header = initcap(header, "header");

  for (i = 0; i < 2; ++i) {

    {
    elementary_tag * beam = NULL;
    beam = inittag (beam, "beam");
    sprintf (buff, "%i", i + 1);                  beam = add_com (beam, initcom(c, "ID", buff, 1));  free(c);
    sprintf (buff, "%.5E", thebeams->benergy[i]); beam = add_com (beam, initcom(c, "energy", buff, 1)); free(c);
    sprintf (buff, "%i", thebeams->bkfcode[i]);   beam = add_com (beam, initcom(c, "KF", buff, 1)); free (c);
    beam = add_com (beam, initcom(c, "name", thebeams->bname[i], 2)); free(c);
    sprintf (buff, "%.5E", thebeams->bmass[i]);   beam = add_com (beam, initcom(c, "mass", buff, 1)); free(c);
    header = add_tag (header, beam);
    freetag(beam);
    }

    {
    elementary_tag * pdf = NULL;
    pdf = inittag (pdf, "strfun");
    sprintf (buff, "%i", i + 1);                       pdf = add_com (pdf, initcom(c, "IDbeam", buff, 1));  free(c);
    pdf = add_com (pdf, initcom(c, "name",    thebeams->pdfname[i], 2)); free(c);
    pdf = add_com (pdf, initcom(c, "version", thebeams->pdfversion[i], 2)); free (c);
    sprintf (buff, "%i", thebeams->pdfgr[i]);          pdf = add_com (pdf, initcom(c, "PDFgr", buff, 1)); free(c);
    sprintf (buff, "%i", thebeams->pdfid[i]);          pdf = add_com (pdf, initcom(c, "PDFid", buff, 1)); free(c);
    header = add_tag (header, pdf);
    freetag(pdf);
    }
  }

  for (i = 0; i < nproc; ++i) {
    int j;
    string_comnd * id;
    process * theproc;
    theproc = read_subproc_info (theproc, inFile);
    sprintf (buff, "%i", i + 1);
    id = initcom(id, "IDprocess", buff, 1);

    {
    elementary_tag * ptag;
    ptag = inittag (ptag, "process");
    sprintf (buff, "%i", i + 1);                 ptag = add_com (ptag, initcom(c, "ID",         buff, 1)); free(c);
                                                 ptag = add_com (ptag, initcom(c, "name",       theproc->name, 2)); free(c);
    sprintf (buff, "%.5E", theproc->crossec);    ptag = add_com (ptag, initcom(c, "CrosSec",    buff, 1)); free(c);
    sprintf (buff, "%.5E", theproc->crossecerr); ptag = add_com (ptag, initcom(c, "CrosSecErr", buff, 1)); free (c);
    sprintf (buff, "%i", theproc->nparton);      ptag = add_com (ptag, initcom(c, "Nparton",    buff, 1)); free(c);
    sprintf (buff, "%i", theproc->master);       ptag = add_com (ptag, initcom(c, "master",     buff, 1)); free(c);
    header = add_tag (header, ptag);
    freetag(ptag);
    nout[i] = theproc->nparton;
    }

    {
    elementary_tag * gtag;
    gtag = inittag (gtag, "generator");
    gtag = add_com (gtag, id);
    gtag = add_com (gtag, initcom(c, "name", "CompHEP", 2)); free(c);
    gtag = add_com (gtag, initcom(c, "version", genversion, 2)); free(c);
    header = add_tag (header, gtag);
    freetag(gtag);
    }

    {
    elementary_tag * etag;
    etag = inittag (etag, "n_event");
    etag = add_com (etag, id);
    sprintf (buff, "%i", theproc->nevents); etag = add_com (etag, initcom(c, "N",    buff, 1)); free(c);
    sprintf (buff, "%.5E", theproc->mult);  etag = add_com (etag, initcom(c, "mult", buff, 1)); free(c);
    sprintf (buff, "%.5E", theproc->maxW);  etag = add_com (etag, initcom(c, "maxW", buff, 1)); free(c);
    sprintf (buff, "%i", theproc->ncuts);   etag = add_com (etag, initcom(c, "CutN", buff, 1)); free (c);
    header = add_tag (header, etag);
    freetag(etag);
    }

    for (j = 0; j < theproc->nparton; ++j) {
      elementary_tag * prtag;
      prtag = inittag (prtag, "parton");
      prtag = add_com (prtag, id);
      if (j < 2) {
        sprintf (buff, "%i", j+1);  prtag = add_com (prtag, initcom(c, "in", buff, 1)); free(c);
        prtag = add_com (prtag, initcom(c, "out", "0", 1)); free(c);
      }
      else {
        prtag = add_com (prtag, initcom(c, "in", "0", 1)); free(c);
        sprintf (buff, "%i", j+1);prtag = add_com (prtag, initcom(c, "out", buff, 1)); free(c);
      }
      sprintf (buff, "%i", theproc->prts[j].kfcode);   prtag = add_com (prtag, initcom(c, "KF", buff, 1)); free(c);
      prtag = add_com (prtag, initcom(c, "name", theproc->prts[j].name, 2)); free(c);
      sprintf (buff, "%.5E", theproc->prts[j].mass);   prtag = add_com (prtag, initcom(c, "mass", buff, 1)); free(c);
      header = add_tag (header, prtag);
      freetag(prtag);
    }

    for (j = 0; j < theproc->ncuts; ++j) {
      elementary_tag * ctag;
      ctag = inittag (ctag, "cut");
      ctag = add_com (ctag, id);
      ctag = add_com (ctag, initcom(c, "name", theproc->cuts[j].name, 2)); free(c);
      switch (theproc->cuts[j].stat) {
        case 1: 
          sprintf (buff, "%.5E", theproc->cuts[j].min); ctag = add_com (ctag, initcom(c, "min", buff, 1)); free(c);
          ctag = add_com (ctag, initcom(c, "reg", "1", 2)); free(c);
          break;
        case 2: 
          sprintf (buff, "%.5E", theproc->cuts[j].max); ctag = add_com (ctag, initcom(c, "max", buff, 1)); free(c);
          ctag = add_com (ctag, initcom(c, "reg", "1", 2)); free(c);
          break;
        case 3: 
          sprintf (buff, "%.5E", theproc->cuts[j].min); ctag = add_com (ctag, initcom(c, "min", buff, 1)); free(c);
          sprintf (buff, "%.5E", theproc->cuts[j].max); ctag = add_com (ctag, initcom(c, "max", buff, 1)); free(c);
          ctag = add_com (ctag, initcom(c, "reg", "1", 2)); free(c);
          break;
        case 4: 
          ctag = add_com (ctag, initcom(c, "filename", theproc->cuts[j].filename, 2)); free(c);
          ctag = add_com (ctag, initcom(c, "reg", "0", 1)); free(c);
          break;
      }
      header = add_tag (header, ctag);
      freetag(ctag);
    }

    {
    elementary_tag * qtag;
    qtag = inittag (qtag, "QCDinfo");
    qtag = add_com (qtag, id);
    sprintf (buff, "%i", theproc->qcdnl);       qtag = add_com (qtag, initcom(c, "NL", buff, 1)); free(c);
    sprintf (buff, "%i", theproc->qcdnflv);     qtag = add_com (qtag, initcom(c, "Nflavour", buff, 1)); free(c);
    sprintf (buff, "%.5E", theproc->qcdlambda); qtag = add_com (qtag, initcom(c, "QCDLambda", buff, 2)); free(c);
    header = add_tag (header, qtag);
    freetag(qtag);
    }

    {
    elementary_tag * ftag;
    ftag = inittag(ftag, "format");
    ftag = add_com (ftag, id);
    ftag = add_com (ftag, initcom(c, "ProcNumber", "i", 2)); free(c);
    ftag = add_com (ftag, initcom(c, "p1.3", "17.10E", 2));  free(c);
    ftag = add_com (ftag, initcom(c, "p2.3", "17.10E", 2));  free(c);
    for (j = 2; j < theproc->nparton; j++) {
      sprintf (buff, "p%i.1", j + 1); ftag = add_com (ftag, initcom(c, buff, "17.10E", 2)); free(c);
      sprintf (buff, "p%i.2", j + 1); ftag = add_com (ftag, initcom(c, buff, "17.10E", 2)); free(c);
      sprintf (buff, "p%i.3", j + 1); ftag = add_com (ftag, initcom(c, buff, "17.10E", 2)); free(c);
    }
    ftag = add_com (ftag, initcom(c, "Qsquared", "10.3E", 2)); free(c);
    ftag = add_com (ftag, initcom(c, "colorchain", "string", 2)); free(c);
    ftag = add_com (ftag, initcom(c, "weight", "10.3E", 2)); free(c);
    printtag (stdout, ftag);
    header = add_tag (header, ftag);
    printtag (stdout, header->tag[header->number_of_tags - 1]);
    freetag(ftag);
    }
  }

  elementary_tag * ttag;
  ttag = inittag(ttag, "total");
  sprintf (buff, "%i", nproc);  ttag = add_com (ttag, initcom(c, "Nproc", buff, 1)); free(c);
  sprintf (buff, "%i", nevents);    ttag = add_com (ttag, initcom(c, "Nevent", buff, 1)); free(c);
  sprintf (buff, "%.5E", fullcs); ttag = add_com (ttag, initcom(c, "CrosSec", buff, 1)); free(c);
  ttag = add_com (ttag, initcom(c, "CrosSecErr", "0.00000E+00", 1)); free (c);
  header = add_tag (header, ttag);
  freetag (ttag);

  writetagbase (outFile, header);

/****************************************************************/
/*         rewriting of events                                  */
  while(NULL != fgets(buff,1000,inFile)) {
    if(NULL != strstr(buff,"#Nproc")) {
       break;
    }
  }
  for (i = 0; i < nevents; ++i) {
    event * ev;
    if (1 == read_event_old_format(ev, inFile, nout, nproc)) {
      write_event_new_format (ev, outFile);
    } else {
      fprintf (stdout, "Warning!...");
      break;
    }
  }
  fclose(inFile);
  fclose(outFile);
  return 0;
}

/*
int read_event_new_format (event * ev, FILE * f, tags * header) {
  char ** eventword = malloc (10 * sizeof (char *));
  int stat = 1;
  int npr;
  char buf[STRSIZE];

  if (fgets (buff, 1000, f)) {
    sscanf (buff, "%i:%[^\n]", &npr, buff);
    npr--;
    pos = 
    stat = event_parser (buff, eventword);
    if (!stat) {
      fprintf (stderr, "Error in event %i\n. Exit!", debug);
      exit (1);
    }
    ev->pvect[3] = atof (eventword[pos[npr][3]]);
    ev->pvect[7] = atof (eventword[pos[npr][7]]);
    for (i = 2; i < Npartons[npr]; i++)
      for (j = 0; j < 3; j++)
        ev->pvect[4 * i + j + 1] = atof (eventword[pos[npr][4 * i + j + 1]]);
  ev->scale = atof (eventword[pos[npr][scalepos]]);
  strcpy (ev->colorchain, eventword[pos[npr][colorpos]]);
  }

  return stat;
}
*/
int read_event_old_format (event * ev, FILE * f, int * nnout, int Nproc) {
  int i;
  int npr;
  int stat = 1;
  char buff[STRSIZE];
  char buf[MAXINOUT + 1][STRSIZE];

  if (fgets (buff, 1000, f)) {
    sscanf (buff, "%i%[^\n]", &npr, buf[0]);
    ev->procnumber = npr;
    npr--;
    if (0 > npr || npr > Nproc-1) {
      fprintf (stderr, "Error! subprocess number is of the limits: Nproc = %i, Nproc_MAX =%i \n", npr, Nproc);
      exit (3);
    }
    for (i = 0; i < 4 * nnout[npr]; i++) ev->pvect[i] = 0;

    sscanf (buf[0], "    %lf %lf%[^a]", ev->pvect + 3, ev->pvect + 7, buf[2]);

    for (i = 2; i < nnout[npr]; ++i) {
      int n = 4 * i;
      sscanf (buf[i], " %lf %lf %lf%[^a]", ev->pvect + n + 1, ev->pvect + n + 2, ev->pvect + n + 3, buf[i + 1]);
    }
    sscanf (buf[i], " %lf%[^a]", &(ev->scale), buff);
    ev->colorchain = strchr(buff,'(');
    ev->nout = nnout[npr];
  }
  else {
    stat = 0;
  }
  return stat;
}


int write_event_new_format (event * ev, FILE * f) {
  int i;

  fprintf (f, "%2d:", ev->procnumber);
  fprintf (f, "%17.10E:%17.10E", ev->pvect[3], ev->pvect[7]);
  for (i = 2; i < ev->nout; i++)
    fprintf (f, ":%17.10E:%17.10E:%17.10E", ev->pvect[4 * i + 1], ev->pvect[4 * i + 2], ev->pvect[4 * i + 3]);
  fprintf (f, ":%10.3E:", ev->scale);
  fprintf (f, "%s:\n",ev->colorchain);
  return 1;
}

process * read_subproc_info (process * pr, FILE * f) {
  int i, j;
  int len;
  int num;
  int err = 0;
  char buff[STRSIZE];
  char * buf;
  char * prname;
  char mass[MAXINOUT][STRSIZE];
  char name[MAXINOUT][PARTNAMLEN];

/*  pr = clean(pr);*/

  if (NULL != pr) pr = NULL;
  pr = malloc (sizeof(process));

  pr->ncuts = 0;
  pr->nevents = 0;
  pr->mult = 0.0;
  pr->maxW = 1.0;
  pr->master = 3;
  pr->genname = "CompHEP";
  pr->genversion = "4.4.4";
  pr->crossecerr = 0.0;

  while ( NULL != fgets (buff, 1000, f) )
    if ( NULL != strstr(buff, "#PROCESS") ) break;

  /* particles */
  sscanf(buff,"#PROCESS  %s%s%[^a]", name[0], name[1], mass[0]);
  buf = strstr(buff, "->");
  len = strlen(buf);
  i = 2; /* exclude -> */
  num = 2;
  while (i < len-1) {
    if (32 == buf[i]) {i++;continue;}
    j = 0;
    while (1) {
      if (32 == buf[i]) {++num;break;}
      name[num][j++] = buf[i++];
    }
  }
  pr->nparton = num;
  prname = malloc (5 * MAXINOUT * sizeof(char));
  prname[0] = '\0';
  for (i = 0; i < num; ++i) {
    if (2 > i) pr->prts[i].stat = -1;
    else pr->prts[i].stat = 1;
    if (2 == i) strcat(prname, "-> ");
    strcat(prname, name[i]);
    strcat(prname, " ");
    pr->prts[i].name = malloc (strlen(name[i]) * sizeof(char));
    strcpy(pr->prts[i].name, name[i]);
    pr->prts[i].kfcode = comphep_particle_name_base (name[i]);
  }
  pr->name = malloc (strlen(prname) * sizeof(char));
  strcpy(pr->name, prname);

  fgets (buff, 1000, f);
  if (fgets (buff, 1000, f)) sscanf(buff,"  SQRT(S) %lf",&(pr->sqrtS));
  else err = 1;
  if (fgets (buff, 1000, f)) sscanf(buff,"  Rapidity(c.m.s) %lf",&(pr->rapidity));
  else err = 1;
  if (fgets (buff, 1000, f)) sscanf(buff,"  StrFun1: %s",pr->pdf[0]);
  else err = 1;
  if (fgets (buff, 1000, f)) sscanf(buff,"  StrFun2: %s",pr->pdf[1]);
  else err = 1;
  if (fgets (buff, 1000, f) ) {
    sscanf(buff,"#MASSES  %[^a]",mass[0]);
    for (i = 0; i < pr->nparton; ++i) {
      double themass;
      sscanf (mass[i], " %lf%[^a]",&themass, mass[i + 1]);
      pr->prts[i].mass = themass;
    }
  }
  else err = 1;
  if (fgets (buff, 1000, f) ) sscanf(buff,"#Cross_section(Width) %lf",&pr->crossec);
  else err = 1;

  if (err) {
    fprintf(stdout, "Error in the file format!");
    exit(3);
  }

  return pr;
}

beams * fillbeams (beams * b, process * pr) {
  int i, j;
  char * strname;
  char * bbname;
  char * temp;
  char bname[STRSIZE];
  char pdfname[STRSIZE];
  double bmom[2];
  double sqrtS, Rapidity;
  double bmass[2] = {0.0,0.0};

  if (NULL != b) b = NULL;
  b = malloc (sizeof(beams));
  if (NULL == pr) return b;

  sqrtS = pr->sqrtS;
  Rapidity = pr->rapidity;

  for (i = 0; i < 2; ++i) {
    double e = (sqrtS * sqrtS + bmass[i] * bmass[i] - bmass[1 - i] * bmass[1 - i]) / (2 * sqrtS);
    double p = sqrt (e * e - bmass[i] * bmass[i]);
    bmom[i] = p * cosh (Rapidity) + e * sinh (Rapidity) * (1 - 2 * i);
    if (bmom[i] * bmom[i] < (10.E-10) * sqrtS) bmom[i] = 0.0;
  }

  for (i = 0; i < 2; ++i) {
    b->benergy[i] = bmom[i];
    if (NULL != strstr (pr->pdf[i],"PDF:")) {    /* hadrons */
      strname = strchr (pr->pdf[i], pr->pdf[i][4]);
      bbname = strchr (pr->pdf[i], '(');
      str_cpy_both(bname, bbname);
      strncpy(pdfname, strname, strlen(strname) - strlen(bbname));


      b->bkfcode[i] = comphep_particle_name_base (bname);
      b->bname[i] = malloc (strlen(bname) * sizeof(char));
      strcpy(b->bname[i], bname);

      for (j = 0; j < strlen(pdfname); ++j) 
        if (isupper(pdfname[j]) ) pdfname[j] = tolower (pdfname[j]);

      if (NULL != strstr(pdfname, "cteq")) {
        b->pdfid[i] = get_pdfsup (pdfname);
        b->pdfgr[i] = 4;
        b->pdfname[i] = malloc (4 * sizeof(char));
        strcpy(b->pdfname[i], "cteq");
        temp = strchr(pdfname, pdfname[4]);
        b->pdfversion[i] = malloc (strlen(temp) * sizeof(char));
        strcpy(b->pdfversion[i], temp);
      }

      if (NULL != strstr(pdfname, "mrst")) {
        b->pdfid[i] = get_pdfsup (pdfname);
        b->pdfgr[i] = 3;
        b->pdfname[i] = malloc (4 * sizeof(char));
        strcpy(b->pdfname[i], "mrst");
        temp = strchr(pdfname, pdfname[4]);
        b->pdfversion[i] = malloc (strlen(temp) * sizeof(char));
        strcpy(b->pdfversion[i], temp);
      }
    }
  }

  return b;
}


void
ShowHelp (int N)
{
  if (0 == N) {
    fprintf (stdout, "Short help:\n"
"  Using: translator.csh {old_format_file} {new_format_file}\n\n"
"The program translates information from a file in the old CompHEP (41.10) to "
"a file in then new format (CompHEP 4.2.0)\n");
  }

  if (1 == N) {
    fprintf (stdout, "Long help:\n"
"Bla-bla-bla...\n");
  }
}
