/* 
   * Copyright (C) 2001-2003, CompHEP Collaboration
   *------------------------------------------------------
   * $Id: tag_reader.c,v 1.8 2003/08/25 09:33:18 sherstnv Exp $
   *
   * $Log: tag_reader.c,v $
   * Revision 1.8  2003/08/25 09:33:18  sherstnv
   * type midstr - middlestr is changed
   *
   * Revision 1.7  2003/06/25 18:46:41  kryukov
   * *** empty log message ***
   *
   * Revision 1.6  2003/06/17 11:15:26  sherstnv
   * An error in memory usage has been fixed.
   *
   * Revision 1.5  2003/04/22 09:01:19  kryukov
   * Apply indent to improve readability of files
   *
   * Revision 1.4  2003/04/20 17:55:02  kryukov
   * Clean up warnings during make
   *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tag_reader.h"
#include "tag_parser.h"

FILE * file;

int char_switch = 0;

int format_error;

char new_symbol (FILE * file)
{
  char sym = fgetc (file);

  if (39 == sym) {     /* 39 -> ' */
    char_switch++;
    char_switch = char_switch % 2;
  }

  if (char_switch)
    return sym;
  while (sym == 32 || sym == 10) /*  32 -> ' ' (space), 10 -> line feed  */
      sym = fgetc (file);
  return sym;
}

static string_comnd * command_reader (string_comnd * c)
{
  long i;
  long len;
  long pos;
  char symb;
  char * buffer;

  len = 0;
  pos = ftell(file);
  symb = new_symbol (file);
  while (',' != symb) {
    symb = new_symbol (file);
    ++len;
  }
  fseek (file, pos, SEEK_SET);

  buffer = malloc (len * sizeof (char));
  for (i = 0; i < len; i++) {
    buffer[i] = new_symbol (file);
  }
  buffer[len] = '\0';
  symb = new_symbol (file);

  format_error = 0;
  if (NULL == strrchr(buffer, '=')) format_error = 1;

  return command_parser (buffer, c);
}

static char * tagname_reader (char * tagname)
{
  long i;
  long len;
  char symb;

  len = 0;
  for (i = 0; i < 2; ++i) {
    symb = new_symbol (file);
  }
  while (':' != symb) {
    symb = new_symbol (file);
    ++len;
  }
  fseek (file, -len, SEEK_CUR);

  tagname = malloc (len * sizeof (char));
  for (i = 0; i < len; i++) {
    symb = new_symbol (file);
    if (symb != ':') {
      tagname[i] = symb;
      tagname[i + 1] = '\0';
    }
  }

  format_error = 0;
  if (0 == strlen(tagname)) format_error = 1;

  return tagname;
}

static elementary_tag * tag_reader (elementary_tag * tag)
{
  int i;
  int len;
  char symb;
  char * line;
  string_comnd * c;

  line = tagname_reader (line);
  if (1 == format_error) {
    fprintf (stderr, 
    "\n***Error! "
    "function: command_reader: "
    "an error with the tag syntax in a tag %s\n", line);
    return tag;
  }
  tag->tagname = realloc (tag->tagname, strlen(line) * sizeof (char));
  strcpy (tag->tagname, line);
  free(line);

  for (i = 0; i < MAX_COMMAND_NUMBER; i++) {
    symb = new_symbol (file);
    if (symb == ';') {
      tag->tagsize = i;
      return tag;
    }
    else
      fseek (file, -1, SEEK_CUR);

    c = command_reader (c);
    if (1 == format_error) {
      fprintf (stderr, "\n***Error! "
      "function: command_reader: "
      "an error with the command syntax in a tag %s\n", tag->tagname);
      exit (1);
    }

    len = strlen(c->name) + strlen(c->value) + 2;
    if (2 == c->type) {
      line = malloc((len + 2) * sizeof(char));
      sprintf (line, "%s='%s'", c->name, c->value);
    } else {
      line = malloc(len * sizeof(char));
      sprintf (line, "%s=%s", c->name, c->value);
    }
    tag->tagsize = addcom (&(tag->commands),tag->tagsize, line);
    free (line);
  }

  if (i == MAX_COMMAND_NUMBER)
  fprintf (stderr, "\nWarning: number of commands in the tag %s exceeds limit (%i commands)!\n",
           tag->tagname, MAX_COMMAND_NUMBER);

  symb = new_symbol (file);
  while (';' != symb) symb = new_symbol (file);
  return tag;
}


extern tags * cup_reader (FILE * inFile, tags * b)
{
  int stop = 0;
  char symbol;

  file = inFile;

  while (!stop) {
    elementary_tag * tag = inittag ("temp");
    tag = tag_reader (tag);
    if (0 == format_error) b = add_tag (b, tag);
    freetag (tag);

    symbol = new_symbol (file);
    if (symbol == ';') stop = 1;
    else fseek (file, -1, SEEK_CUR);
  }
  return b;
}
